from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardInstanceProviderDeploymentsConfigVaultsListOutput, DashboardInstanceProviderDeploymentsConfigVaultsListOutput, mapDashboardInstanceProviderDeploymentsConfigVaultsListQuery, DashboardInstanceProviderDeploymentsConfigVaultsListQuery, mapDashboardInstanceProviderDeploymentsConfigVaultsGetOutput, DashboardInstanceProviderDeploymentsConfigVaultsGetOutput, mapDashboardInstanceProviderDeploymentsConfigVaultsCreateOutput, DashboardInstanceProviderDeploymentsConfigVaultsCreateOutput, mapDashboardInstanceProviderDeploymentsConfigVaultsCreateBody, DashboardInstanceProviderDeploymentsConfigVaultsCreateBody, mapDashboardInstanceProviderDeploymentsConfigVaultsUpdateOutput, DashboardInstanceProviderDeploymentsConfigVaultsUpdateOutput, mapDashboardInstanceProviderDeploymentsConfigVaultsUpdateBody, DashboardInstanceProviderDeploymentsConfigVaultsUpdateBody, mapDashboardInstanceProviderDeploymentsConfigVaultsDeleteOutput, DashboardInstanceProviderDeploymentsConfigVaultsDeleteOutput

class MetorialProviderDeploymentsConfigVaultsEndpoint(BaseMetorialEndpoint):
    """A config vault is a saved, reusable set of configuration values. Use vaults to store credentials once and apply them to multiple deployments without re-entering."""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None, status: Optional[Union[str, List[str]]] = None, id: Optional[Union[str, List[str]]] = None, provider_id: Optional[Union[str, List[str]]] = None, provider_deployment_id: Optional[Union[str, List[str]]] = None, provider_config_id: Optional[Union[str, List[str]]] = None, provider_config_vault_id: Optional[Union[str, List[str]]] = None, search: Optional[str] = None) -> DashboardInstanceProviderDeploymentsConfigVaultsListOutput:
        """
    List provider config vaults
    Returns a paginated list of provider config vaults.

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param status: Optional[Union[str, List[str]]] (optional)
    :param id: Optional[Union[str, List[str]]] (optional)
    :param provider_id: Optional[Union[str, List[str]]] (optional)
    :param provider_deployment_id: Optional[Union[str, List[str]]] (optional)
    :param provider_config_id: Optional[Union[str, List[str]]] (optional)
    :param provider_config_vault_id: Optional[Union[str, List[str]]] (optional)
    :param search: Optional[str] (optional)
    :return: DashboardInstanceProviderDeploymentsConfigVaultsListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order
        if status is not None:
            query_dict["status"] = status
        if id is not None:
            query_dict["id"] = id
        if provider_id is not None:
            query_dict["provider_id"] = provider_id
        if provider_deployment_id is not None:
            query_dict["provider_deployment_id"] = provider_deployment_id
        if provider_config_id is not None:
            query_dict["provider_config_id"] = provider_config_id
        if provider_config_vault_id is not None:
            query_dict["provider_config_vault_id"] = provider_config_vault_id
        if search is not None:
            query_dict["search"] = search

        request = MetorialRequest(
            path=['provider-config-vaults'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardInstanceProviderDeploymentsConfigVaultsListOutput.from_dict)

    def get(self, provider_config_vault_id: str) -> DashboardInstanceProviderDeploymentsConfigVaultsGetOutput:
        """
    Get provider config vault
    Retrieves a specific provider config vault by ID.

    :param provider_config_vault_id: str
    :return: DashboardInstanceProviderDeploymentsConfigVaultsGetOutput
    """
        request = MetorialRequest(
            path=['provider-config-vaults', provider_config_vault_id]
        )
        return self._get(request).transform(mapDashboardInstanceProviderDeploymentsConfigVaultsGetOutput.from_dict)

    def create(self, *, provider_id: str, name: str, value: Dict[str, Any], provider_deployment_id: Optional[str] = None, description: Optional[str] = None, metadata: Optional[Dict[str, Any]] = None) -> DashboardInstanceProviderDeploymentsConfigVaultsCreateOutput:
        """
    Create provider config vault
    Creates a new provider config vault.

    :param provider_id: str
    :param provider_deployment_id: Optional[str] (optional)
    :param name: str
    :param description: Optional[str] (optional)
    :param metadata: Optional[Dict[str, Any]] (optional)
    :param value: Dict[str, Any]
    :return: DashboardInstanceProviderDeploymentsConfigVaultsCreateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        body_dict["provider_id"] = provider_id
        if provider_deployment_id is not None:
            body_dict["provider_deployment_id"] = provider_deployment_id
        body_dict["name"] = name
        if description is not None:
            body_dict["description"] = description
        if metadata is not None:
            body_dict["metadata"] = metadata
        body_dict["value"] = value

        request = MetorialRequest(
            path=['provider-config-vaults'],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardInstanceProviderDeploymentsConfigVaultsCreateOutput.from_dict)

    def update(self, provider_config_vault_id: str, *, name: Optional[str] = None, description: Optional[str] = None, metadata: Optional[Dict[str, Any]] = None) -> DashboardInstanceProviderDeploymentsConfigVaultsUpdateOutput:
        """
    Update provider config vault
    Updates a specific provider config vault.

    :param provider_config_vault_id: str
    :param name: Optional[str] (optional)
    :param description: Optional[str] (optional)
    :param metadata: Optional[Dict[str, Any]] (optional)
    :return: DashboardInstanceProviderDeploymentsConfigVaultsUpdateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        if name is not None:
            body_dict["name"] = name
        if description is not None:
            body_dict["description"] = description
        if metadata is not None:
            body_dict["metadata"] = metadata

        request = MetorialRequest(
            path=['provider-config-vaults', provider_config_vault_id],
            body=body_dict
        )
        return self._patch(request).transform(mapDashboardInstanceProviderDeploymentsConfigVaultsUpdateOutput.from_dict)

    def delete(self, provider_config_vault_id: str) -> DashboardInstanceProviderDeploymentsConfigVaultsDeleteOutput:
        """
    Delete provider config vault
    Permanently deletes a provider config vault.

    :param provider_config_vault_id: str
    :return: DashboardInstanceProviderDeploymentsConfigVaultsDeleteOutput
    """
        request = MetorialRequest(
            path=['provider-config-vaults', provider_config_vault_id]
        )
        return self._delete(request).transform(mapDashboardInstanceProviderDeploymentsConfigVaultsDeleteOutput.from_dict)


from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardScmAccountsPreviewOutputItems:
  provider: str
  name: str
  identifier: str
  external_id: str


@dataclass
class DashboardScmAccountsPreviewOutput:
  object: str
  items: List[DashboardScmAccountsPreviewOutputItems]


class mapDashboardScmAccountsPreviewOutputItems:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardScmAccountsPreviewOutputItems:
    return DashboardScmAccountsPreviewOutputItems(
      provider=data.get("provider"),
      name=data.get("name"),
      identifier=data.get("identifier"),
      external_id=data.get("external_id"),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardScmAccountsPreviewOutputItems, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardScmAccountsPreviewOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardScmAccountsPreviewOutput:
    return DashboardScmAccountsPreviewOutput(
      object=data.get("object"),
      items=[
        mapDashboardScmAccountsPreviewOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardScmAccountsPreviewOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class DashboardScmAccountsPreviewQuery:
  installation_id: str


class mapDashboardScmAccountsPreviewQuery:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardScmAccountsPreviewQuery:
    return DashboardScmAccountsPreviewQuery(installation_id=data.get("installation_id"))

  @staticmethod
  def to_dict(
    value: Union[DashboardScmAccountsPreviewQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceScmAccountsPreviewOutputAccounts:
    object: str
    provider: str
    external_id: str
    name: str
    identifier: str
@dataclass
class DashboardInstanceScmAccountsPreviewOutput:
    object: str
    accounts: List[DashboardInstanceScmAccountsPreviewOutputAccounts]


class mapDashboardInstanceScmAccountsPreviewOutputAccounts:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceScmAccountsPreviewOutputAccounts:
        return DashboardInstanceScmAccountsPreviewOutputAccounts(
        object=data.get('object'),
        provider=data.get('provider'),
        external_id=data.get('external_id'),
        name=data.get('name'),
        identifier=data.get('identifier')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceScmAccountsPreviewOutputAccounts, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceScmAccountsPreviewOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceScmAccountsPreviewOutput:
        return DashboardInstanceScmAccountsPreviewOutput(
        object=data.get('object'),
        accounts=[mapDashboardInstanceScmAccountsPreviewOutputAccounts.from_dict(item) for item in data.get('accounts', []) if item]
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceScmAccountsPreviewOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardInstanceScmAccountsPreviewBody:
    installation_id: str


class mapDashboardInstanceScmAccountsPreviewBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceScmAccountsPreviewBody:
        return DashboardInstanceScmAccountsPreviewBody(
        installation_id=data.get('installation_id')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceScmAccountsPreviewBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapDashboardInstanceCustomServersDeploymentsListOutput,
  DashboardInstanceCustomServersDeploymentsListOutput,
  mapDashboardInstanceCustomServersDeploymentsListQuery,
  DashboardInstanceCustomServersDeploymentsListQuery,
  mapDashboardInstanceCustomServersDeploymentsGetOutput,
  DashboardInstanceCustomServersDeploymentsGetOutput,
)


class MetorialDashboardInstanceCustomServersDeploymentsEndpoint(BaseMetorialEndpoint):
  """Manager custom server deployments"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def list(
    self,
    instance_id: str,
    custom_server_id: str,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None,
    version_id: Optional[Union[str, List[str]]] = None
  ) -> DashboardInstanceCustomServersDeploymentsListOutput:
    """
    List custom server deployments
    List all custom server deployments

    :param instance_id: str
    :param custom_server_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param version_id: Optional[Union[str, List[str]]] (optional)
    :return: DashboardInstanceCustomServersDeploymentsListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order
    if version_id is not None:
      query_dict["version_id"] = version_id

    request = MetorialRequest(
      path=[
        "dashboard",
        "instances",
        instance_id,
        "custom-servers",
        custom_server_id,
        "deployments",
      ],
      query=query_dict,
    )
    return self._get(request).transform(
      mapDashboardInstanceCustomServersDeploymentsListOutput.from_dict
    )

  def get(
    self, instance_id: str, custom_server_id: str, custom_server_deployment_id: str
  ) -> DashboardInstanceCustomServersDeploymentsGetOutput:
    """
    Get custom server deployment
    Get information for a specific custom server deployment

    :param instance_id: str
    :param custom_server_id: str
    :param custom_server_deployment_id: str
    :return: DashboardInstanceCustomServersDeploymentsGetOutput
    """
    request = MetorialRequest(
      path=[
        "dashboard",
        "instances",
        instance_id,
        "custom-servers",
        custom_server_id,
        "deployments",
        custom_server_deployment_id,
      ]
    )
    return self._get(request).transform(
      mapDashboardInstanceCustomServersDeploymentsGetOutput.from_dict
    )



from typing import Any, Dict, List, Optional, Union
from metorial_util_endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardOrganizationsOauthAuthorizationsListOutput, DashboardOrganizationsOauthAuthorizationsListOutput, mapDashboardOrganizationsOauthAuthorizationsListQuery, DashboardOrganizationsOauthAuthorizationsListQuery, mapDashboardOrganizationsOauthAuthorizationsGetOutput, DashboardOrganizationsOauthAuthorizationsGetOutput, mapDashboardOrganizationsOauthAuthorizationsRevokeOutput, DashboardOrganizationsOauthAuthorizationsRevokeOutput

class MetorialDashboardOrganizationsOauthAuthorizationsEndpoint(BaseMetorialEndpoint):
    """Inspect and revoke OAuth authorizations for an organization"""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, organization_id: str, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None, status: Optional[Union[str, List[str]]] = None, installation_id: Optional[Union[str, List[str]]] = None, app_id: Optional[Union[str, List[str]]] = None) -> DashboardOrganizationsOauthAuthorizationsListOutput:
        """
    List organization OAuth authorizations
    Returns a paginated list of OAuth authorizations for the organization.

    :param organization_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param status: Optional[Union[str, List[str]]] (optional)
    :param installation_id: Optional[Union[str, List[str]]] (optional)
    :param app_id: Optional[Union[str, List[str]]] (optional)
    :return: DashboardOrganizationsOauthAuthorizationsListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order
        if status is not None:
            query_dict["status"] = status
        if installation_id is not None:
            query_dict["installation_id"] = installation_id
        if app_id is not None:
            query_dict["app_id"] = app_id

        request = MetorialRequest(
            path=['dashboard', 'organizations', organization_id, 'oauth', 'authorizations'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardOrganizationsOauthAuthorizationsListOutput.from_dict)

    def get(self, organization_id: str, oauth_authorization_id: str) -> DashboardOrganizationsOauthAuthorizationsGetOutput:
        """
    Get organization OAuth authorization
    Retrieves a specific OAuth authorization for the organization.

    :param organization_id: str
    :param oauth_authorization_id: str
    :return: DashboardOrganizationsOauthAuthorizationsGetOutput
    """
        request = MetorialRequest(
            path=['dashboard', 'organizations', organization_id, 'oauth', 'authorizations', oauth_authorization_id]
        )
        return self._get(request).transform(mapDashboardOrganizationsOauthAuthorizationsGetOutput.from_dict)

    def revoke(self, organization_id: str, oauth_authorization_id: str) -> DashboardOrganizationsOauthAuthorizationsRevokeOutput:
        """
    Revoke organization OAuth authorization
    Revokes a specific OAuth authorization for the organization.

    :param organization_id: str
    :param oauth_authorization_id: str
    :return: DashboardOrganizationsOauthAuthorizationsRevokeOutput
    """
        request = MetorialRequest(
            path=['dashboard', 'organizations', organization_id, 'oauth', 'authorizations', oauth_authorization_id, 'revoke']
        )
        return self._post(request).transform(mapDashboardOrganizationsOauthAuthorizationsRevokeOutput.from_dict)


from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials:
    object: str
    id: str
    type: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes:
    object: str
    id: str
    scope: str
    name: str
    description: Optional[str] = None
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod:
    object: str
    id: str
    type: str
    key: str
    name: str
    capabilities: Dict[str, Any]
    provider_id: str
    provider_specification_id: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    input_schema: Optional[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema] = None
    output_schema: Optional[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema] = None
    scopes: Optional[List[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes]] = None
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig:
    object: str
    id: str
    type: str
    source: str
    status: str
    is_default: bool
    provider_id: str
    auth_method: ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    deployment_preview: Optional[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview] = None
    credentials: Optional[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials] = None
@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateOutput:
    object: str
    id: str
    note: str
    auth_config: ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig
    provider_id: str
    auth_method_id: str
    created_at: datetime
    ip: Optional[str] = None
    user_agent: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    provider_deployment_id: Optional[str] = None
    credentials_id: Optional[str] = None
    value: Optional[Dict[str, Any]] = None
    expires_at: Optional[datetime] = None


class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes(
        object=data.get('object'),
        id=data.get('id'),
        scope=data.get('scope'),
        name=data.get('name'),
        description=data.get('description')
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        key=data.get('key'),
        name=data.get('name'),
        description=data.get('description'),
        capabilities=data.get('capabilities'),
        input_schema=mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodInputSchema.from_dict(data.get('input_schema')) if data.get('input_schema') else None,
        output_schema=mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodOutputSchema.from_dict(data.get('output_schema')) if data.get('output_schema') else None,
        scopes=[mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethodScopes.from_dict(item) for item in data.get('scopes', []) if item],
        provider_id=data.get('provider_id'),
        provider_specification_id=data.get('provider_specification_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig:
        return ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        source=data.get('source'),
        status=data.get('status'),
        is_default=data.get('is_default'),
        provider_id=data.get('provider_id'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        deployment_preview=mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigDeploymentPreview.from_dict(data.get('deployment_preview')) if data.get('deployment_preview') else None,
        credentials=mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigCredentials.from_dict(data.get('credentials')) if data.get('credentials') else None,
        auth_method=mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfigAuthMethod.from_dict(data.get('auth_method')) if data.get('auth_method') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsAuthConfigsExportsCreateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateOutput:
        return ProviderDeploymentsAuthConfigsExportsCreateOutput(
        object=data.get('object'),
        id=data.get('id'),
        note=data.get('note'),
        ip=data.get('ip'),
        user_agent=data.get('user_agent'),
        metadata=data.get('metadata'),
        auth_config=mapProviderDeploymentsAuthConfigsExportsCreateOutputAuthConfig.from_dict(data.get('auth_config')) if data.get('auth_config') else None,
        provider_id=data.get('provider_id'),
        provider_deployment_id=data.get('provider_deployment_id'),
        auth_method_id=data.get('auth_method_id'),
        credentials_id=data.get('credentials_id'),
        value=data.get('value'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        expires_at=datetime.fromisoformat(data.get('expires_at').replace('Z', '+00:00')) if data.get('expires_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class ProviderDeploymentsAuthConfigsExportsCreateBody:
    provider_auth_config_id: str
    note: str
    metadata: Optional[Dict[str, Any]] = None


class mapProviderDeploymentsAuthConfigsExportsCreateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsAuthConfigsExportsCreateBody:
        return ProviderDeploymentsAuthConfigsExportsCreateBody(
        provider_auth_config_id=data.get('provider_auth_config_id'),
        note=data.get('note'),
        metadata=data.get('metadata')
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsAuthConfigsExportsCreateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class IdentitiesCredentialsListOutputItems:
    object: str
    id: str
    status: str
    identity_id: str
    provider_id: str
    created_at: datetime
    updated_at: datetime
    deployment_id: Optional[str] = None
    config_id: Optional[str] = None
    auth_config_id: Optional[str] = None
    delegation_config_id: Optional[str] = None
@dataclass
class IdentitiesCredentialsListOutputPagination:
    has_more_before: bool
    has_more_after: bool
@dataclass
class IdentitiesCredentialsListOutput:
    items: List[IdentitiesCredentialsListOutputItems]
    pagination: IdentitiesCredentialsListOutputPagination


class mapIdentitiesCredentialsListOutputItems:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> IdentitiesCredentialsListOutputItems:
        return IdentitiesCredentialsListOutputItems(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        identity_id=data.get('identity_id'),
        provider_id=data.get('provider_id'),
        deployment_id=data.get('deployment_id'),
        config_id=data.get('config_id'),
        auth_config_id=data.get('auth_config_id'),
        delegation_config_id=data.get('delegation_config_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[IdentitiesCredentialsListOutputItems, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapIdentitiesCredentialsListOutputPagination:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> IdentitiesCredentialsListOutputPagination:
        return IdentitiesCredentialsListOutputPagination(
        has_more_before=data.get('has_more_before'),
        has_more_after=data.get('has_more_after')
        )

    @staticmethod
    def to_dict(value: Union[IdentitiesCredentialsListOutputPagination, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapIdentitiesCredentialsListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> IdentitiesCredentialsListOutput:
        return IdentitiesCredentialsListOutput(
        items=[mapIdentitiesCredentialsListOutputItems.from_dict(item) for item in data.get('items', []) if item],
        pagination=mapIdentitiesCredentialsListOutputPagination.from_dict(data.get('pagination')) if data.get('pagination') else None
        )

    @staticmethod
    def to_dict(value: Union[IdentitiesCredentialsListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class IdentitiesCredentialsListQueryCreatedAt:
    gt: Optional[datetime] = None
    lt: Optional[datetime] = None
@dataclass
class IdentitiesCredentialsListQueryUpdatedAt:
    gt: Optional[datetime] = None
    lt: Optional[datetime] = None
@dataclass
class IdentitiesCredentialsListQuery:
    limit: Optional[float] = None
    after: Optional[str] = None
    before: Optional[str] = None
    cursor: Optional[str] = None
    order: Optional[str] = None
    status: Optional[Union[str, List[str]]] = None
    id: Optional[Union[str, List[str]]] = None
    agent_id: Optional[Union[str, List[str]]] = None
    actor_id: Optional[Union[str, List[str]]] = None
    identity_id: Optional[Union[str, List[str]]] = None
    provider_id: Optional[Union[str, List[str]]] = None
    provider_deployment_id: Optional[Union[str, List[str]]] = None
    provider_config_id: Optional[Union[str, List[str]]] = None
    provider_auth_config_id: Optional[Union[str, List[str]]] = None
    created_at: Optional[IdentitiesCredentialsListQueryCreatedAt] = None
    updated_at: Optional[IdentitiesCredentialsListQueryUpdatedAt] = None


class mapIdentitiesCredentialsListQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> IdentitiesCredentialsListQuery:
        return IdentitiesCredentialsListQuery(
        limit=data.get('limit'),
        after=data.get('after'),
        before=data.get('before'),
        cursor=data.get('cursor'),
        order=data.get('order'),
        status=data.get('status'),
        id=data.get('id'),
        agent_id=data.get('agent_id'),
        actor_id=data.get('actor_id'),
        identity_id=data.get('identity_id'),
        provider_id=data.get('provider_id'),
        provider_deployment_id=data.get('provider_deployment_id'),
        provider_config_id=data.get('provider_config_id'),
        provider_auth_config_id=data.get('provider_auth_config_id'),
        created_at=mapIdentitiesCredentialsListQueryCreatedAt.from_dict(data.get('created_at')) if data.get('created_at') else None,
        updated_at=mapIdentitiesCredentialsListQueryUpdatedAt.from_dict(data.get('updated_at')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[IdentitiesCredentialsListQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardScmInstallationsCreateOutput:
  object: str
  authorization_url: str


class mapDashboardScmInstallationsCreateOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardScmInstallationsCreateOutput:
    return DashboardScmInstallationsCreateOutput(
      object=data.get("object"), authorization_url=data.get("authorization_url")
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardScmInstallationsCreateOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class DashboardScmInstallationsCreateBody:
  provider: str
  redirect_url: str


class mapDashboardScmInstallationsCreateBody:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardScmInstallationsCreateBody:
    return DashboardScmInstallationsCreateBody(
      provider=data.get("provider"), redirect_url=data.get("redirect_url")
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardScmInstallationsCreateBody, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ManagementInstanceServersImplementationsDeleteOutputServerVariant:
  object: str
  id: str
  identifier: str
  server_id: str
  source: Dict[str, Any]
  created_at: datetime


@dataclass
class ManagementInstanceServersImplementationsDeleteOutputServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceServersImplementationsDeleteOutput:
  object: str
  id: str
  status: str
  is_default: bool
  is_ephemeral: bool
  name: str
  metadata: Dict[str, Any]
  server_variant: ManagementInstanceServersImplementationsDeleteOutputServerVariant
  server: ManagementInstanceServersImplementationsDeleteOutputServer
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None
  get_launch_params: Optional[str] = None


class mapManagementInstanceServersImplementationsDeleteOutputServerVariant:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServersImplementationsDeleteOutputServerVariant:
    return ManagementInstanceServersImplementationsDeleteOutputServerVariant(
      object=data.get("object"),
      id=data.get("id"),
      identifier=data.get("identifier"),
      server_id=data.get("server_id"),
      source=data.get("source"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServersImplementationsDeleteOutputServerVariant,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServersImplementationsDeleteOutputServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServersImplementationsDeleteOutputServer:
    return ManagementInstanceServersImplementationsDeleteOutputServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServersImplementationsDeleteOutputServer, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServersImplementationsDeleteOutput:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServersImplementationsDeleteOutput:
    return ManagementInstanceServersImplementationsDeleteOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      is_default=data.get("is_default"),
      is_ephemeral=data.get("is_ephemeral"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      get_launch_params=data.get("get_launch_params"),
      server_variant=mapManagementInstanceServersImplementationsDeleteOutputServerVariant.from_dict(
        data.get("server_variant")
      )
      if data.get("server_variant")
      else None,
      server=mapManagementInstanceServersImplementationsDeleteOutputServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServersImplementationsDeleteOutput, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



"""
Typed endpoint classes for better IDE support
"""

from typing import TYPE_CHECKING, Any

from metorial._endpoint import MetorialEndpointManager

if TYPE_CHECKING:
  from metorial._generated.pulsar.endpoints.provider_oauth_connections_authentications import (
    MetorialProviderOauthConnectionsAuthenticationsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.provider_oauth_connections_profiles import (
    MetorialProviderOauthConnectionsProfilesEndpoint,
  )
  from metorial._generated.pulsar.endpoints.provider_oauth_sessions import (
    MetorialProviderOauthSessionsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.server_runs import (
    MetorialServerRunsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.servers_capabilities import (
    MetorialServersCapabilitiesEndpoint,
  )
  from metorial._generated.pulsar.endpoints.servers_deployments import (
    MetorialServersDeploymentsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.servers_implementations import (
    MetorialServersImplementationsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.servers_variants import (
    MetorialServersVariantsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.servers_versions import (
    MetorialServersVersionsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.sessions_connections import (
    MetorialSessionsConnectionsEndpoint,
  )
  from metorial._generated.pulsar.endpoints.sessions_messages import (
    MetorialSessionsMessagesEndpoint,
  )


if TYPE_CHECKING:
  # Import base endpoint classes for type checking only
  from metorial._generated.pulsar.endpoints.provider_oauth_connections import (
    MetorialProviderOauthConnectionsEndpoint as _MetorialProviderOauthConnectionsEndpointBase,
  )
  from metorial._generated.pulsar.endpoints.servers import (
    MetorialServersEndpoint as _MetorialServersEndpointBase,
  )
  from metorial._generated.pulsar.endpoints.sessions import (
    MetorialSessionsEndpoint as _MetorialSessionsEndpointBase,
  )

  # For type checkers, make the typed endpoints inherit from base to get all methods
  _TypedServersBase = _MetorialServersEndpointBase
  _TypedSessionsBase = _MetorialSessionsEndpointBase
  _TypedProviderOauthConnectionsBase = _MetorialProviderOauthConnectionsEndpointBase
else:
  # At runtime, just use object as base
  _TypedServersBase = object
  _TypedSessionsBase = object
  _TypedProviderOauthConnectionsBase = object


class TypedMetorialServersEndpoint(_TypedServersBase):
  """Typed servers endpoint with all sub-endpoints"""

  # Type annotations for IDE support - sub-endpoints
  variants: "MetorialServersVariantsEndpoint"
  versions: "MetorialServersVersionsEndpoint"
  deployments: "MetorialServersDeploymentsEndpoint"
  implementations: "MetorialServersImplementationsEndpoint"
  capabilities: "MetorialServersCapabilitiesEndpoint"
  runs: "MetorialServerRunsEndpoint"

  def __init__(self, manager: MetorialEndpointManager):
    # Import here to avoid circular imports
    from metorial._generated.pulsar.endpoints.server_run_errors import (
      MetorialServerRunErrorsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.server_runs import (
      MetorialServerRunsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.servers import MetorialServersEndpoint
    from metorial._generated.pulsar.endpoints.servers_capabilities import (
      MetorialServersCapabilitiesEndpoint,
    )
    from metorial._generated.pulsar.endpoints.servers_deployments import (
      MetorialServersDeploymentsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.servers_implementations import (
      MetorialServersImplementationsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.servers_variants import (
      MetorialServersVariantsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.servers_versions import (
      MetorialServersVersionsEndpoint,
    )

    # Create the base servers endpoint to inherit its methods
    self._base_servers = MetorialServersEndpoint(manager)

    # Add sub-endpoints
    self.variants = MetorialServersVariantsEndpoint(manager)
    self.versions = MetorialServersVersionsEndpoint(manager)
    self.deployments = MetorialServersDeploymentsEndpoint(manager)
    self.implementations = MetorialServersImplementationsEndpoint(manager)
    self.capabilities = MetorialServersCapabilitiesEndpoint(manager)

    self.runs = MetorialServerRunsEndpoint(manager)
    setattr(self.runs, "errors", MetorialServerRunErrorsEndpoint(manager))  # noqa: B010

  def __getattr__(self, name: str) -> Any:
    """Delegate unknown attributes to the base servers endpoint"""
    return getattr(self._base_servers, name)


class TypedMetorialSessionsEndpoint(_TypedSessionsBase):
  """Typed sessions endpoint with sub-endpoints"""

  # Type annotations for IDE support
  messages: "MetorialSessionsMessagesEndpoint"
  connections: "MetorialSessionsConnectionsEndpoint"

  def __init__(self, manager: MetorialEndpointManager):
    from metorial._generated.pulsar.endpoints.sessions import MetorialSessionsEndpoint
    from metorial._generated.pulsar.endpoints.sessions_connections import (
      MetorialSessionsConnectionsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.sessions_messages import (
      MetorialSessionsMessagesEndpoint,
    )

    # Create the base sessions endpoint to inherit its methods
    self._base_sessions = MetorialSessionsEndpoint(manager)

    # Add sub-endpoints
    self.messages = MetorialSessionsMessagesEndpoint(manager)
    self.connections = MetorialSessionsConnectionsEndpoint(manager)

  def __getattr__(self, name: str) -> Any:
    """Delegate unknown attributes to the base sessions endpoint"""
    return getattr(self._base_sessions, name)


class TypedMetorialProviderOauthConnectionsEndpoint(_TypedProviderOauthConnectionsBase):
  """Typed connections endpoint with nested authentications and profiles"""

  # Type annotations for IDE support
  authentications: "MetorialProviderOauthConnectionsAuthenticationsEndpoint"
  profiles: "MetorialProviderOauthConnectionsProfilesEndpoint"

  def __init__(self, base_endpoint: object, manager: MetorialEndpointManager) -> None:
    from metorial._generated.pulsar.endpoints.provider_oauth_connections_authentications import (
      MetorialProviderOauthConnectionsAuthenticationsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.provider_oauth_connections_profiles import (
      MetorialProviderOauthConnectionsProfilesEndpoint,
    )

    # Store base endpoint for delegation
    self._base = base_endpoint

    # Add sub-endpoints
    self.authentications = MetorialProviderOauthConnectionsAuthenticationsEndpoint(
      manager
    )
    self.profiles = MetorialProviderOauthConnectionsProfilesEndpoint(manager)

  def __getattr__(self, name: str) -> Any:
    """Delegate unknown attributes to the base connections endpoint"""
    return getattr(self._base, name)


class TypedMetorialProviderOauthEndpoint:
  """Typed provider OAuth endpoint with sub-endpoints"""

  # Type annotations for IDE support
  connections: "TypedMetorialProviderOauthConnectionsEndpoint"
  sessions: "MetorialProviderOauthSessionsEndpoint"
  profiles: "MetorialProviderOauthConnectionsProfilesEndpoint"
  authentications: "MetorialProviderOauthConnectionsAuthenticationsEndpoint"

  def __init__(self, manager: MetorialEndpointManager):
    from metorial._generated.pulsar.endpoints.provider_oauth_connections import (
      MetorialProviderOauthConnectionsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.provider_oauth_connections_authentications import (
      MetorialProviderOauthConnectionsAuthenticationsEndpoint,
    )
    from metorial._generated.pulsar.endpoints.provider_oauth_connections_profiles import (
      MetorialProviderOauthConnectionsProfilesEndpoint,
    )
    from metorial._generated.pulsar.endpoints.provider_oauth_sessions import (
      MetorialProviderOauthSessionsEndpoint,
    )

    base_connections = MetorialProviderOauthConnectionsEndpoint(manager)
    self.connections = TypedMetorialProviderOauthConnectionsEndpoint(
      base_connections, manager
    )
    self.sessions = MetorialProviderOauthSessionsEndpoint(manager)
    self.profiles = MetorialProviderOauthConnectionsProfilesEndpoint(manager)
    self.authentications = MetorialProviderOauthConnectionsAuthenticationsEndpoint(
      manager
    )


__all__ = [
  "TypedMetorialServersEndpoint",
  "TypedMetorialSessionsEndpoint",
  "TypedMetorialProviderOauthEndpoint",
  "TypedMetorialProviderOauthConnectionsEndpoint",
]



"""
LangChain integration for Metorial tools.

Example:
    from langchain_anthropic import ChatAnthropic
    from langchain.agents import create_react_agent
    from metorial import Metorial, metorial_langchain

    metorial = Metorial(api_key="...")

    session = await metorial.connect(
        adapter=metorial_langchain(),
        providers=[{"provider_deployment_id": "deployment-id"}],
    )
    agent = create_react_agent(ChatAnthropic(), session.tools())
    result = await agent.ainvoke({"messages": [("user", "Search for news")]})
"""

from __future__ import annotations

import json
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
  from metorial._client import ProviderSession

try:
  from langchain_core.tools import StructuredTool
except ImportError:
  StructuredTool = None


def create_langchain_tools(session: ProviderSession) -> list[Any]:
  """Convert Metorial session tools to LangChain tools.

  Args:
      session: Active Metorial ProviderSession

  Returns:
      List of LangChain StructuredTool objects

  Raises:
      ImportError: If langchain-core is not installed
  """
  if StructuredTool is None:
    raise ImportError(
      "langchain-core is required for LangChain integration. "
      "Install it with: pip install langchain-core"
    )

  tools = []
  tool_manager = session.tool_manager

  if tool_manager is None:
    return tools

  for tool in tool_manager.get_tools():
    # Get the tool's parameters schema
    schema = tool.get_parameters_as("json-schema") or {}

    # Capture tool_name in closure to avoid late binding issues
    def make_tool_fn(tool_name: str):
      async def tool_fn(**kwargs: Any) -> str:
        filtered_kwargs = {k: v for k, v in kwargs.items() if v is not None}
        try:
          result = await tool_manager.execute_tool(tool_name, filtered_kwargs)
          if hasattr(result, "model_dump"):
            result = result.model_dump()
          return json.dumps(result, ensure_ascii=False, default=str)
        except Exception as e:
          return json.dumps({"error": str(e)}, ensure_ascii=False)

      return tool_fn

    # Create the LangChain tool (async only — use ainvoke/astream with the agent)
    lc_tool = StructuredTool.from_function(
      coroutine=make_tool_fn(tool.name),
      name=tool.name,
      description=tool.description or f"Tool: {tool.name}",
      args_schema=_schema_to_pydantic(tool.name, schema),
    )
    tools.append(lc_tool)

  return tools


def _schema_to_pydantic(name: str, schema: dict[str, Any]) -> Any:
  """Convert JSON schema to Pydantic model for LangChain."""
  try:
    from pydantic import Field, create_model
  except ImportError:
    return None

  properties = schema.get("properties", {})
  required = set(schema.get("required", []))

  fields = {}
  for prop_name, prop_schema in properties.items():
    prop_type = _json_type_to_python(prop_schema.get("type", "string"))
    description = prop_schema.get("description", "")
    default = ... if prop_name in required else None

    fields[prop_name] = (prop_type, Field(default=default, description=description))

  return create_model(f"{name}Input", **fields)


def _json_type_to_python(json_type: str) -> type:
  """Convert JSON schema type to Python type."""
  type_map = {
    "string": str,
    "integer": int,
    "number": float,
    "boolean": bool,
    "array": list,
    "object": dict,
  }
  return type_map.get(json_type, str)


__all__ = ["create_langchain_tools"]



from typing import Any, Dict, List, Optional, Union
from metorial_util_endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardOrganizationsMembersListOutput, DashboardOrganizationsMembersListOutput, mapDashboardOrganizationsMembersListQuery, DashboardOrganizationsMembersListQuery, mapDashboardOrganizationsMembersGetOutput, DashboardOrganizationsMembersGetOutput, mapDashboardOrganizationsMembersDeleteOutput, DashboardOrganizationsMembersDeleteOutput, mapDashboardOrganizationsMembersUpdateOutput, DashboardOrganizationsMembersUpdateOutput, mapDashboardOrganizationsMembersUpdateBody, DashboardOrganizationsMembersUpdateBody

class MetorialManagementOrganizationMembersEndpoint(BaseMetorialEndpoint):
    """Read and write organization member information"""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None, team_id: Optional[Union[str, List[str]]] = None) -> DashboardOrganizationsMembersListOutput:
        """
    List organization members
    List all organization members

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param team_id: Optional[Union[str, List[str]]] (optional)
    :return: DashboardOrganizationsMembersListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order
        if team_id is not None:
            query_dict["team_id"] = team_id

        request = MetorialRequest(
            path=['organization', 'members'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardOrganizationsMembersListOutput.from_dict)

    def get(self, member_id: str) -> DashboardOrganizationsMembersGetOutput:
        """
    Get organization member
    Get the information of a specific organization member

    :param member_id: str
    :return: DashboardOrganizationsMembersGetOutput
    """
        request = MetorialRequest(
            path=['organization', 'members', member_id]
        )
        return self._get(request).transform(mapDashboardOrganizationsMembersGetOutput.from_dict)

    def delete(self, member_id: str) -> DashboardOrganizationsMembersDeleteOutput:
        """
    Delete organization member
    Remove an organization member

    :param member_id: str
    :return: DashboardOrganizationsMembersDeleteOutput
    """
        request = MetorialRequest(
            path=['organization', 'members', member_id]
        )
        return self._delete(request).transform(mapDashboardOrganizationsMembersDeleteOutput.from_dict)

    def update(self, member_id: str, *, role: str) -> DashboardOrganizationsMembersUpdateOutput:
        """
    Update organization member
    Update the role of an organization member

    :param member_id: str
    :param role: str
    :return: DashboardOrganizationsMembersUpdateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        body_dict["role"] = role

        request = MetorialRequest(
            path=['organization', 'members', member_id],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardOrganizationsMembersUpdateOutput.from_dict)


from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardOrganizationsTeamsListOutputItemsProjectsProject:
  object: str
  id: str
  status: str
  slug: str
  name: str
  organization_id: str
  created_at: datetime
  updated_at: datetime


@dataclass
class DashboardOrganizationsTeamsListOutputItemsProjectsRolesRole:
  object: str
  id: str
  organization_id: str
  name: str
  slug: str
  permissions: List[str]
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class DashboardOrganizationsTeamsListOutputItemsProjectsRoles:
  id: str
  role: DashboardOrganizationsTeamsListOutputItemsProjectsRolesRole
  created_at: datetime
  updated_at: datetime


@dataclass
class DashboardOrganizationsTeamsListOutputItemsProjects:
  id: str
  created_at: datetime
  updated_at: datetime
  project: DashboardOrganizationsTeamsListOutputItemsProjectsProject
  roles: List[DashboardOrganizationsTeamsListOutputItemsProjectsRoles]


@dataclass
class DashboardOrganizationsTeamsListOutputItems:
  object: str
  id: str
  organization_id: str
  name: str
  slug: str
  projects: List[DashboardOrganizationsTeamsListOutputItemsProjects]
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class DashboardOrganizationsTeamsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class DashboardOrganizationsTeamsListOutput:
  items: List[DashboardOrganizationsTeamsListOutputItems]
  pagination: DashboardOrganizationsTeamsListOutputPagination


class mapDashboardOrganizationsTeamsListOutputItemsProjectsProject:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardOrganizationsTeamsListOutputItemsProjectsProject:
    return DashboardOrganizationsTeamsListOutputItemsProjectsProject(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      slug=data.get("slug"),
      name=data.get("name"),
      organization_id=data.get("organization_id"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardOrganizationsTeamsListOutputItemsProjectsProject, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardOrganizationsTeamsListOutputItemsProjectsRolesRole:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardOrganizationsTeamsListOutputItemsProjectsRolesRole:
    return DashboardOrganizationsTeamsListOutputItemsProjectsRolesRole(
      object=data.get("object"),
      id=data.get("id"),
      organization_id=data.get("organization_id"),
      name=data.get("name"),
      slug=data.get("slug"),
      description=data.get("description"),
      permissions=data.get("permissions", []),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardOrganizationsTeamsListOutputItemsProjectsRolesRole, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardOrganizationsTeamsListOutputItemsProjectsRoles:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardOrganizationsTeamsListOutputItemsProjectsRoles:
    return DashboardOrganizationsTeamsListOutputItemsProjectsRoles(
      id=data.get("id"),
      role=mapDashboardOrganizationsTeamsListOutputItemsProjectsRolesRole.from_dict(
        data.get("role")
      )
      if data.get("role")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardOrganizationsTeamsListOutputItemsProjectsRoles, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardOrganizationsTeamsListOutputItemsProjects:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardOrganizationsTeamsListOutputItemsProjects:
    return DashboardOrganizationsTeamsListOutputItemsProjects(
      id=data.get("id"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      project=mapDashboardOrganizationsTeamsListOutputItemsProjectsProject.from_dict(
        data.get("project")
      )
      if data.get("project")
      else None,
      roles=[
        mapDashboardOrganizationsTeamsListOutputItemsProjectsRoles.from_dict(item)
        for item in data.get("roles", [])
        if item
      ],
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardOrganizationsTeamsListOutputItemsProjects, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardOrganizationsTeamsListOutputItems:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsTeamsListOutputItems:
    return DashboardOrganizationsTeamsListOutputItems(
      object=data.get("object"),
      id=data.get("id"),
      organization_id=data.get("organization_id"),
      name=data.get("name"),
      slug=data.get("slug"),
      description=data.get("description"),
      projects=[
        mapDashboardOrganizationsTeamsListOutputItemsProjects.from_dict(item)
        for item in data.get("projects", [])
        if item
      ],
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardOrganizationsTeamsListOutputItems, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardOrganizationsTeamsListOutputPagination:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardOrganizationsTeamsListOutputPagination:
    return DashboardOrganizationsTeamsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardOrganizationsTeamsListOutputPagination, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardOrganizationsTeamsListOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsTeamsListOutput:
    return DashboardOrganizationsTeamsListOutput(
      items=[
        mapDashboardOrganizationsTeamsListOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
      pagination=mapDashboardOrganizationsTeamsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardOrganizationsTeamsListOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class DashboardOrganizationsTeamsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None


class mapDashboardOrganizationsTeamsListQuery:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsTeamsListQuery:
    return DashboardOrganizationsTeamsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardOrganizationsTeamsListQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ManagementInstanceSessionsGetOutputClientSecret:
  object: str
  type: str
  id: str
  secret: str
  expires_at: datetime


@dataclass
class ManagementInstanceSessionsGetOutputServerDeploymentsServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls:
  sse: str
  streamable_http: str


@dataclass
class ManagementInstanceSessionsGetOutputServerDeployments:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: ManagementInstanceSessionsGetOutputServerDeploymentsServer
  connection_urls: ManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls
  name: Optional[str] = None
  oauth_session_id: Optional[str] = None
  description: Optional[str] = None


@dataclass
class ManagementInstanceSessionsGetOutputUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class ManagementInstanceSessionsGetOutput:
  object: str
  id: str
  status: str
  connection_status: str
  client_secret: ManagementInstanceSessionsGetOutputClientSecret
  server_deployments: List[ManagementInstanceSessionsGetOutputServerDeployments]
  usage: ManagementInstanceSessionsGetOutputUsage
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime


class mapManagementInstanceSessionsGetOutputClientSecret:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsGetOutputClientSecret:
    return ManagementInstanceSessionsGetOutputClientSecret(
      object=data.get("object"),
      type=data.get("type"),
      id=data.get("id"),
      secret=data.get("secret"),
      expires_at=parse_iso_datetime(data.get("expires_at"))
      if data.get("expires_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceSessionsGetOutputClientSecret, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsGetOutputServerDeploymentsServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsGetOutputServerDeploymentsServer:
    return ManagementInstanceSessionsGetOutputServerDeploymentsServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsGetOutputServerDeploymentsServer, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls:
    return ManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls(
      sse=data.get("sse"), streamable_http=data.get("streamable_http")
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsGetOutputServerDeployments:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsGetOutputServerDeployments:
    return ManagementInstanceSessionsGetOutputServerDeployments(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      oauth_session_id=data.get("oauth_session_id"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapManagementInstanceSessionsGetOutputServerDeploymentsServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      connection_urls=mapManagementInstanceSessionsGetOutputServerDeploymentsConnectionUrls.from_dict(
        data.get("connection_urls")
      )
      if data.get("connection_urls")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsGetOutputServerDeployments, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsGetOutputUsage:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsGetOutputUsage:
    return ManagementInstanceSessionsGetOutputUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceSessionsGetOutputUsage, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsGetOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsGetOutput:
    return ManagementInstanceSessionsGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      connection_status=data.get("connection_status"),
      client_secret=mapManagementInstanceSessionsGetOutputClientSecret.from_dict(
        data.get("client_secret")
      )
      if data.get("client_secret")
      else None,
      server_deployments=[
        mapManagementInstanceSessionsGetOutputServerDeployments.from_dict(item)
        for item in data.get("server_deployments", [])
        if item
      ],
      usage=mapManagementInstanceSessionsGetOutputUsage.from_dict(data.get("usage"))
      if data.get("usage")
      else None,
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceSessionsGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class FileLinksListOutputItems:
    object: str
    id: str
    file_id: str
    url: str
    created_at: datetime
    expires_at: Optional[datetime] = None
@dataclass
class FileLinksListOutputPagination:
    has_more_before: bool
    has_more_after: bool
@dataclass
class FileLinksListOutput:
    items: List[FileLinksListOutputItems]
    pagination: FileLinksListOutputPagination


class mapFileLinksListOutputItems:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> FileLinksListOutputItems:
        return FileLinksListOutputItems(
        object=data.get('object'),
        id=data.get('id'),
        file_id=data.get('file_id'),
        url=data.get('url'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        expires_at=datetime.fromisoformat(data.get('expires_at').replace('Z', '+00:00')) if data.get('expires_at') else None
        )

    @staticmethod
    def to_dict(value: Union[FileLinksListOutputItems, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapFileLinksListOutputPagination:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> FileLinksListOutputPagination:
        return FileLinksListOutputPagination(
        has_more_before=data.get('has_more_before'),
        has_more_after=data.get('has_more_after')
        )

    @staticmethod
    def to_dict(value: Union[FileLinksListOutputPagination, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapFileLinksListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> FileLinksListOutput:
        return FileLinksListOutput(
        items=[mapFileLinksListOutputItems.from_dict(item) for item in data.get('items', []) if item],
        pagination=mapFileLinksListOutputPagination.from_dict(data.get('pagination')) if data.get('pagination') else None
        )

    @staticmethod
    def to_dict(value: Union[FileLinksListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class FileLinksListQuery:
    limit: Optional[float] = None
    after: Optional[str] = None
    before: Optional[str] = None
    cursor: Optional[str] = None
    order: Optional[str] = None
    file_id: Optional[str] = None


class mapFileLinksListQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> FileLinksListQuery:
        return FileLinksListQuery(
        limit=data.get('limit'),
        after=data.get('after'),
        before=data.get('before'),
        cursor=data.get('cursor'),
        order=data.get('order'),
        file_id=data.get('file_id')
        )

    @staticmethod
    def to_dict(value: Union[FileLinksListQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapDashboardScmAccountsPreviewOutput,
  DashboardScmAccountsPreviewOutput,
  mapDashboardScmAccountsPreviewQuery,
  DashboardScmAccountsPreviewQuery,
)


class MetorialDashboardScmAccountsEndpoint(BaseMetorialEndpoint):
  """Read and write SCM repository information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def preview(
    self, organization_id: str, *, installation_id: str
  ) -> DashboardScmAccountsPreviewOutput:
    """
    List SCM Repositories
    List SCM accounts for all organizations the user is a member of

    :param organization_id: str
    :param installation_id: str
    :return: DashboardScmAccountsPreviewOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    query_dict["installation_id"] = installation_id

    request = MetorialRequest(
      path=[
        "dashboard",
        "organizations",
        organization_id,
        "scm",
        "accounts",
        "preview",
      ],
      query=query_dict,
    )
    return self._get(request).transform(mapDashboardScmAccountsPreviewOutput.from_dict)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceIdentityActorsListOutputItems:
    object: str
    id: str
    type: str
    status: str
    name: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    agent_id: Optional[str] = None
@dataclass
class DashboardInstanceIdentityActorsListOutputPagination:
    has_more_before: bool
    has_more_after: bool
@dataclass
class DashboardInstanceIdentityActorsListOutput:
    items: List[DashboardInstanceIdentityActorsListOutputItems]
    pagination: DashboardInstanceIdentityActorsListOutputPagination


class mapDashboardInstanceIdentityActorsListOutputItems:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceIdentityActorsListOutputItems:
        return DashboardInstanceIdentityActorsListOutputItems(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        status=data.get('status'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        agent_id=data.get('agent_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceIdentityActorsListOutputItems, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceIdentityActorsListOutputPagination:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceIdentityActorsListOutputPagination:
        return DashboardInstanceIdentityActorsListOutputPagination(
        has_more_before=data.get('has_more_before'),
        has_more_after=data.get('has_more_after')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceIdentityActorsListOutputPagination, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceIdentityActorsListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceIdentityActorsListOutput:
        return DashboardInstanceIdentityActorsListOutput(
        items=[mapDashboardInstanceIdentityActorsListOutputItems.from_dict(item) for item in data.get('items', []) if item],
        pagination=mapDashboardInstanceIdentityActorsListOutputPagination.from_dict(data.get('pagination')) if data.get('pagination') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceIdentityActorsListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardInstanceIdentityActorsListQueryCreatedAt:
    gt: Optional[datetime] = None
    lt: Optional[datetime] = None
@dataclass
class DashboardInstanceIdentityActorsListQueryUpdatedAt:
    gt: Optional[datetime] = None
    lt: Optional[datetime] = None
@dataclass
class DashboardInstanceIdentityActorsListQuery:
    limit: Optional[float] = None
    after: Optional[str] = None
    before: Optional[str] = None
    cursor: Optional[str] = None
    order: Optional[str] = None
    search: Optional[str] = None
    status: Optional[Union[str, List[str]]] = None
    id: Optional[Union[str, List[str]]] = None
    agent_id: Optional[Union[str, List[str]]] = None
    created_at: Optional[DashboardInstanceIdentityActorsListQueryCreatedAt] = None
    updated_at: Optional[DashboardInstanceIdentityActorsListQueryUpdatedAt] = None


class mapDashboardInstanceIdentityActorsListQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceIdentityActorsListQuery:
        return DashboardInstanceIdentityActorsListQuery(
        limit=data.get('limit'),
        after=data.get('after'),
        before=data.get('before'),
        cursor=data.get('cursor'),
        order=data.get('order'),
        search=data.get('search'),
        status=data.get('status'),
        id=data.get('id'),
        agent_id=data.get('agent_id'),
        created_at=mapDashboardInstanceIdentityActorsListQueryCreatedAt.from_dict(data.get('created_at')) if data.get('created_at') else None,
        updated_at=mapDashboardInstanceIdentityActorsListQueryUpdatedAt.from_dict(data.get('updated_at')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceIdentityActorsListQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ServersImplementationsUpdateOutputServerVariant:
  object: str
  id: str
  identifier: str
  server_id: str
  source: Dict[str, Any]
  created_at: datetime


@dataclass
class ServersImplementationsUpdateOutputServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ServersImplementationsUpdateOutput:
  object: str
  id: str
  status: str
  name: str
  metadata: Dict[str, Any]
  server_variant: ServersImplementationsUpdateOutputServerVariant
  server: ServersImplementationsUpdateOutputServer
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None
  get_launch_params: Optional[str] = None


class mapServersImplementationsUpdateOutputServerVariant:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ServersImplementationsUpdateOutputServerVariant:
    return ServersImplementationsUpdateOutputServerVariant(
      object=data.get("object"),
      id=data.get("id"),
      identifier=data.get("identifier"),
      server_id=data.get("server_id"),
      source=data.get("source"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersImplementationsUpdateOutputServerVariant, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersImplementationsUpdateOutputServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersImplementationsUpdateOutputServer:
    return ServersImplementationsUpdateOutputServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersImplementationsUpdateOutputServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersImplementationsUpdateOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersImplementationsUpdateOutput:
    return ServersImplementationsUpdateOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      get_launch_params=data.get("get_launch_params"),
      server_variant=mapServersImplementationsUpdateOutputServerVariant.from_dict(
        data.get("server_variant")
      )
      if data.get("server_variant")
      else None,
      server=mapServersImplementationsUpdateOutputServer.from_dict(data.get("server"))
      if data.get("server")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersImplementationsUpdateOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class ServersImplementationsUpdateBody:
  name: Optional[str] = None
  description: Optional[str] = None
  metadata: Optional[Dict[str, Any]] = None
  get_launch_params: Optional[str] = None


class mapServersImplementationsUpdateBody:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersImplementationsUpdateBody:
    return ServersImplementationsUpdateBody(
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      get_launch_params=data.get("get_launch_params"),
    )

  @staticmethod
  def to_dict(
    value: Union[ServersImplementationsUpdateBody, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardInstanceCustomProvidersDeploymentsListOutput, DashboardInstanceCustomProvidersDeploymentsListOutput, mapDashboardInstanceCustomProvidersDeploymentsListQuery, DashboardInstanceCustomProvidersDeploymentsListQuery, mapDashboardInstanceCustomProvidersDeploymentsGetOutput, DashboardInstanceCustomProvidersDeploymentsGetOutput, mapDashboardInstanceCustomProvidersDeploymentsGetLogsOutput, DashboardInstanceCustomProvidersDeploymentsGetLogsOutput

class MetorialDashboardInstanceCustomProvidersDeploymentsEndpoint(BaseMetorialEndpoint):
    """Deployments track the build and deployment process of custom provider versions. View deployment status and logs."""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, instance_id: str, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None, status: Optional[Union[str, List[str]]] = None, id: Optional[Union[str, List[str]]] = None, custom_provider_version_id: Optional[Union[str, List[str]]] = None, custom_provider_id: Optional[Union[str, List[str]]] = None) -> DashboardInstanceCustomProvidersDeploymentsListOutput:
        """
    List custom provider deployments
    Returns a paginated list of deployments for a custom provider.

    :param instance_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param status: Optional[Union[str, List[str]]] (optional)
    :param id: Optional[Union[str, List[str]]] (optional)
    :param custom_provider_version_id: Optional[Union[str, List[str]]] (optional)
    :param custom_provider_id: Optional[Union[str, List[str]]] (optional)
    :return: DashboardInstanceCustomProvidersDeploymentsListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order
        if status is not None:
            query_dict["status"] = status
        if id is not None:
            query_dict["id"] = id
        if custom_provider_version_id is not None:
            query_dict["custom_provider_version_id"] = custom_provider_version_id
        if custom_provider_id is not None:
            query_dict["custom_provider_id"] = custom_provider_id

        request = MetorialRequest(
            path=['dashboard', 'instances', instance_id, 'custom-provider-deployments'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardInstanceCustomProvidersDeploymentsListOutput.from_dict)

    def get(self, instance_id: str, custom_provider_deployment_id: str) -> DashboardInstanceCustomProvidersDeploymentsGetOutput:
        """
    Get custom provider deployment
    Retrieves a specific deployment.

    :param instance_id: str
    :param custom_provider_deployment_id: str
    :return: DashboardInstanceCustomProvidersDeploymentsGetOutput
    """
        request = MetorialRequest(
            path=['dashboard', 'instances', instance_id, 'custom-provider-deployments', custom_provider_deployment_id]
        )
        return self._get(request).transform(mapDashboardInstanceCustomProvidersDeploymentsGetOutput.from_dict)

    def get_logs(self, instance_id: str, custom_provider_deployment_id: str) -> DashboardInstanceCustomProvidersDeploymentsGetLogsOutput:
        """
    Get deployment logs
    Retrieves the build and deployment logs for a deployment.

    :param instance_id: str
    :param custom_provider_deployment_id: str
    :return: DashboardInstanceCustomProvidersDeploymentsGetLogsOutput
    """
        request = MetorialRequest(
            path=['dashboard', 'instances', instance_id, 'custom-provider-deployments', custom_provider_deployment_id, 'logs']
        )
        return self._get(request).transform(mapDashboardInstanceCustomProvidersDeploymentsGetLogsOutput.from_dict)


from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardUsageTimelineOutputTimelineEntries:
    ts: datetime
    count: float
@dataclass
class DashboardUsageTimelineOutputTimeline:
    entity_id: str
    entity_type: str
    owner_id: str
    entries: List[DashboardUsageTimelineOutputTimelineEntries]
@dataclass
class DashboardUsageTimelineOutput:
    object: str
    timeline: List[DashboardUsageTimelineOutputTimeline]


class mapDashboardUsageTimelineOutputTimelineEntries:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardUsageTimelineOutputTimelineEntries:
        return DashboardUsageTimelineOutputTimelineEntries(
        ts=datetime.fromisoformat(data.get('ts').replace('Z', '+00:00')) if data.get('ts') else None,
        count=data.get('count')
        )

    @staticmethod
    def to_dict(value: Union[DashboardUsageTimelineOutputTimelineEntries, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardUsageTimelineOutputTimeline:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardUsageTimelineOutputTimeline:
        return DashboardUsageTimelineOutputTimeline(
        entity_id=data.get('entity_id'),
        entity_type=data.get('entity_type'),
        owner_id=data.get('owner_id'),
        entries=[mapDashboardUsageTimelineOutputTimelineEntries.from_dict(item) for item in data.get('entries', []) if item]
        )

    @staticmethod
    def to_dict(value: Union[DashboardUsageTimelineOutputTimeline, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardUsageTimelineOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardUsageTimelineOutput:
        return DashboardUsageTimelineOutput(
        object=data.get('object'),
        timeline=[mapDashboardUsageTimelineOutputTimeline.from_dict(item) for item in data.get('timeline', []) if item]
        )

    @staticmethod
    def to_dict(value: Union[DashboardUsageTimelineOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardUsageTimelineQueryEntities:
    type: str
    id: str
@dataclass
class DashboardUsageTimelineQueryInterval:
    unit: str
    count: float
@dataclass
class DashboardUsageTimelineQuery:
    entities: List[DashboardUsageTimelineQueryEntities]
    from_: datetime
    to: datetime
    interval: DashboardUsageTimelineQueryInterval


class mapDashboardUsageTimelineQueryEntities:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardUsageTimelineQueryEntities:
        return DashboardUsageTimelineQueryEntities(
        type=data.get('type'),
        id=data.get('id')
        )

    @staticmethod
    def to_dict(value: Union[DashboardUsageTimelineQueryEntities, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardUsageTimelineQueryInterval:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardUsageTimelineQueryInterval:
        return DashboardUsageTimelineQueryInterval(
        unit=data.get('unit'),
        count=data.get('count')
        )

    @staticmethod
    def to_dict(value: Union[DashboardUsageTimelineQueryInterval, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardUsageTimelineQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardUsageTimelineQuery:
        return DashboardUsageTimelineQuery(
        entities=[mapDashboardUsageTimelineQueryEntities.from_dict(item) for item in data.get('entities', []) if item],
        from_=datetime.fromisoformat(data.get('from').replace('Z', '+00:00')) if data.get('from') else None,
        to=datetime.fromisoformat(data.get('to').replace('Z', '+00:00')) if data.get('to') else None,
        interval=mapDashboardUsageTimelineQueryInterval.from_dict(data.get('interval')) if data.get('interval') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardUsageTimelineQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

